@@ -19,3 +19,5 @@
 
 set -ex
 cargo clippy --all-targets --workspace --features avro,pyarrow -- -D warnings
+cd datafusion-cli
+cargo clippy --all-targets --all-features -- -D warnings
