@@ -25,12 +25,12 @@ pub mod aggregate_statistics;
 pub mod coalesce_batches;
 pub mod combine_partial_final_agg;
 pub mod dist_enforcement;
-pub mod global_sort_selection;
 pub mod join_selection;
 pub mod optimizer;
 pub mod pipeline_checker;
 pub mod pruning;
 pub mod repartition;
+pub mod replace_repartition_execs;
 pub mod sort_enforcement;
 mod sort_pushdown;
 mod utils;
