@@ -24,25 +24,32 @@ use std::sync::Arc;
 use std::task::{Context, Poll};
 
 use super::expressions::PhysicalSortExpr;
-use super::{ColumnStatistics, RecordBatchStream, SendableRecordBatchStream, Statistics};
+use super::{
+    ColumnStatistics, DisplayAs, RecordBatchStream, SendableRecordBatchStream, Statistics,
+};
+
 use crate::physical_plan::{
     metrics::{BaselineMetrics, ExecutionPlanMetricsSet, MetricsSet},
     Column, DisplayFormatType, EquivalenceProperties, ExecutionPlan, Partitioning,
-    PhysicalExpr,
 };
+
 use arrow::compute::filter_record_batch;
 use arrow::datatypes::{DataType, SchemaRef};
 use arrow::record_batch::RecordBatch;
 use datafusion_common::cast::as_boolean_array;
 use datafusion_common::{DataFusionError, Result};
+use datafusion_execution::TaskContext;
 use datafusion_expr::Operator;
 use datafusion_physical_expr::expressions::BinaryExpr;
-use datafusion_physical_expr::{split_conjunction, AnalysisContext};
+use datafusion_physical_expr::intervals::is_operator_supported;
+use datafusion_physical_expr::utils::collect_columns;
+use datafusion_physical_expr::{
+    analyze, split_conjunction, AnalysisContext, ExprBoundaries,
+    OrderingEquivalenceProperties, PhysicalExpr,
+};
 
-use log::trace;
-
-use datafusion_execution::TaskContext;
 use futures::stream::{Stream, StreamExt};
+use log::trace;
 
 /// FilterExec evaluates a boolean predicate against all input batches to determine which rows to
 /// include in its output batches.
@@ -85,6 +92,20 @@ impl FilterExec {
     }
 }
 
+impl DisplayAs for FilterExec {
+    fn fmt_as(
+        &self,
+        t: DisplayFormatType,
+        f: &mut std::fmt::Formatter,
+    ) -> std::fmt::Result {
+        match t {
+            DisplayFormatType::Default | DisplayFormatType::Verbose => {
+                write!(f, "FilterExec: {}", self.predicate)
+            }
+        }
+    }
+}
+
 impl ExecutionPlan for FilterExec {
     /// Return a reference to Any that can be used for downcasting
     fn as_any(&self) -> &dyn Any {
@@ -132,6 +153,10 @@ impl ExecutionPlan for FilterExec {
         input_properties
     }
 
+    fn ordering_equivalence_properties(&self) -> OrderingEquivalenceProperties {
+        self.input.ordering_equivalence_properties()
+    }
+
     fn with_new_children(
         self: Arc<Self>,
         children: Vec<Arc<dyn ExecutionPlan>>,
@@ -157,18 +182,6 @@ impl ExecutionPlan for FilterExec {
         }))
     }
 
-    fn fmt_as(
-        &self,
-        t: DisplayFormatType,
-        f: &mut std::fmt::Formatter,
-    ) -> std::fmt::Result {
-        match t {
-            DisplayFormatType::Default | DisplayFormatType::Verbose => {
-                write!(f, "FilterExec: {}", self.predicate)
-            }
-        }
-    }
-
     fn metrics(&self) -> Option<MetricsSet> {
         Some(self.metrics.clone_inner())
     }
@@ -176,52 +189,88 @@ impl ExecutionPlan for FilterExec {
     /// The output statistics of a filtering operation can be estimated if the
     /// predicate's selectivity value can be determined for the incoming data.
     fn statistics(&self) -> Statistics {
-        let input_stats = self.input.statistics();
-        let starter_ctx =
-            AnalysisContext::from_statistics(self.input.schema().as_ref(), &input_stats);
+        let predicate = self.predicate();
 
-        let analysis_ctx = self.predicate.analyze(starter_ctx);
-
-        match analysis_ctx.boundaries {
-            Some(boundaries) => {
-                // Build back the column level statistics from the boundaries inside the
-                // analysis context. It is possible that these are going to be different
-                // than the input statistics, especially when a comparison is made inside
-                // the predicate expression (e.g. `col1 > 100`).
-                let column_statistics = analysis_ctx
-                    .column_boundaries
-                    .iter()
-                    .map(|boundary| match boundary {
-                        Some(boundary) => ColumnStatistics {
-                            min_value: Some(boundary.min_value.clone()),
-                            max_value: Some(boundary.max_value.clone()),
-                            ..Default::default()
-                        },
-                        None => ColumnStatistics::default(),
-                    })
-                    .collect();
-
-                Statistics {
-                    num_rows: input_stats.num_rows.zip(boundaries.selectivity).map(
-                        |(num_rows, selectivity)| {
-                            (num_rows as f64 * selectivity).ceil() as usize
-                        },
-                    ),
-                    total_byte_size: input_stats
-                        .total_byte_size
-                        .zip(boundaries.selectivity)
-                        .map(|(num_rows, selectivity)| {
-                            (num_rows as f64 * selectivity).ceil() as usize
-                        }),
-                    column_statistics: Some(column_statistics),
-                    ..Default::default()
-                }
+        if let Some(binary) = predicate.as_any().downcast_ref::<BinaryExpr>() {
+            let columns = collect_columns(predicate);
+            if !is_operator_supported(binary.op()) || columns.is_empty() {
+                return Statistics::default();
             }
-            None => Statistics::default(),
+        }
+
+        let input_stats = self.input.statistics();
+        let input_column_stats = match input_stats.column_statistics {
+            Some(stats) => stats,
+            None => return Statistics::default(),
+        };
+
+        let starter_ctx =
+            AnalysisContext::from_statistics(&self.input.schema(), &input_column_stats);
+
+        let analysis_ctx = match analyze(predicate, starter_ctx) {
+            Ok(ctx) => ctx,
+            Err(_) => return Statistics::default(),
+        };
+
+        let selectivity = analysis_ctx.selectivity.unwrap_or(1.0);
+
+        let num_rows = input_stats
+            .num_rows
+            .map(|num| (num as f64 * selectivity).ceil() as usize);
+        let total_byte_size = input_stats
+            .total_byte_size
+            .map(|size| (size as f64 * selectivity).ceil() as usize);
+
+        let column_statistics = if let Some(analysis_boundaries) = analysis_ctx.boundaries
+        {
+            collect_new_statistics(input_column_stats, selectivity, analysis_boundaries)
+        } else {
+            input_column_stats
+        };
+
+        Statistics {
+            num_rows,
+            total_byte_size,
+            column_statistics: Some(column_statistics),
+            is_exact: Default::default(),
         }
     }
 }
 
+/// This function ensures that all bounds in the `ExprBoundaries` vector are
+/// converted to closed bounds. If a lower/upper bound is initially open, it
+/// is adjusted by using the next/previous value for its data type to convert
+/// it into a closed bound.
+fn collect_new_statistics(
+    input_column_stats: Vec<ColumnStatistics>,
+    selectivity: f64,
+    analysis_boundaries: Vec<ExprBoundaries>,
+) -> Vec<ColumnStatistics> {
+    let nonempty_columns = selectivity > 0.0;
+    analysis_boundaries
+        .into_iter()
+        .enumerate()
+        .map(
+            |(
+                idx,
+                ExprBoundaries {
+                    interval,
+                    distinct_count,
+                    ..
+                },
+            )| {
+                let closed_interval = interval.close_bounds();
+                ColumnStatistics {
+                    null_count: input_column_stats[idx].null_count,
+                    max_value: nonempty_columns.then_some(closed_interval.upper.value),
+                    min_value: nonempty_columns.then_some(closed_interval.lower.value),
+                    distinct_count,
+                }
+            },
+        )
+        .collect()
+}
+
 /// The FilterExec streams wraps the input iterator and applies the predicate expression to
 /// determine which rows to include in its output batches
 struct FilterExecStream {
@@ -474,40 +523,6 @@ mod tests {
         let statistics = filter.statistics();
         assert_eq!(statistics.num_rows, Some(25));
         assert_eq!(statistics.total_byte_size, Some(25 * bytes_per_row));
-
-        Ok(())
-    }
-
-    #[tokio::test]
-    async fn test_filter_statistics_column_level_basic_expr() -> Result<()> {
-        // Table:
-        //      a: min=1, max=100
-        let schema = Schema::new(vec![Field::new("a", DataType::Int32, false)]);
-        let input = Arc::new(StatisticsExec::new(
-            Statistics {
-                num_rows: Some(100),
-                column_statistics: Some(vec![ColumnStatistics {
-                    min_value: Some(ScalarValue::Int32(Some(1))),
-                    max_value: Some(ScalarValue::Int32(Some(100))),
-                    ..Default::default()
-                }]),
-                ..Default::default()
-            },
-            schema.clone(),
-        ));
-
-        // a <= 25
-        let predicate: Arc<dyn PhysicalExpr> =
-            binary(col("a", &schema)?, Operator::LtEq, lit(25i32), &schema)?;
-
-        // WHERE a <= 25
-        let filter: Arc<dyn ExecutionPlan> =
-            Arc::new(FilterExec::try_new(predicate, input)?);
-
-        let statistics = filter.statistics();
-
-        // a must be in [1, 25] range now!
-        assert_eq!(statistics.num_rows, Some(25));
         assert_eq!(
             statistics.column_statistics,
             Some(vec![ColumnStatistics {
@@ -612,7 +627,6 @@ mod tests {
             binary(col("a", &schema)?, Operator::GtEq, lit(10i32), &schema)?,
             b_gt_5,
         )?);
-
         let statistics = filter.statistics();
         // On a uniform distribution, only fifteen rows will satisfy the
         // filter that 'a' proposed (a >= 10 AND a <= 25) (15/100) and only
@@ -630,7 +644,7 @@ mod tests {
                     ..Default::default()
                 },
                 ColumnStatistics {
-                    min_value: Some(ScalarValue::Int32(Some(45))),
+                    min_value: Some(ScalarValue::Int32(Some(46))),
                     max_value: Some(ScalarValue::Int32(Some(50))),
                     ..Default::default()
                 }
@@ -668,4 +682,309 @@ mod tests {
 
         Ok(())
     }
+
+    #[tokio::test]
+    async fn test_filter_statistics_multiple_columns() -> Result<()> {
+        // Table:
+        //      a: min=1, max=100
+        //      b: min=1, max=3
+        //      c: min=1000.0  max=1100.0
+        let schema = Schema::new(vec![
+            Field::new("a", DataType::Int32, false),
+            Field::new("b", DataType::Int32, false),
+            Field::new("c", DataType::Float32, false),
+        ]);
+        let input = Arc::new(StatisticsExec::new(
+            Statistics {
+                num_rows: Some(1000),
+                total_byte_size: Some(4000),
+                column_statistics: Some(vec![
+                    ColumnStatistics {
+                        min_value: Some(ScalarValue::Int32(Some(1))),
+                        max_value: Some(ScalarValue::Int32(Some(100))),
+                        ..Default::default()
+                    },
+                    ColumnStatistics {
+                        min_value: Some(ScalarValue::Int32(Some(1))),
+                        max_value: Some(ScalarValue::Int32(Some(3))),
+                        ..Default::default()
+                    },
+                    ColumnStatistics {
+                        min_value: Some(ScalarValue::Float32(Some(1000.0))),
+                        max_value: Some(ScalarValue::Float32(Some(1100.0))),
+                        ..Default::default()
+                    },
+                ]),
+                ..Default::default()
+            },
+            schema,
+        ));
+        // WHERE a<=53 AND (b=3 AND (c<=1075.0 AND a>b))
+        let predicate = Arc::new(BinaryExpr::new(
+            Arc::new(BinaryExpr::new(
+                Arc::new(Column::new("a", 0)),
+                Operator::LtEq,
+                Arc::new(Literal::new(ScalarValue::Int32(Some(53)))),
+            )),
+            Operator::And,
+            Arc::new(BinaryExpr::new(
+                Arc::new(BinaryExpr::new(
+                    Arc::new(Column::new("b", 1)),
+                    Operator::Eq,
+                    Arc::new(Literal::new(ScalarValue::Int32(Some(3)))),
+                )),
+                Operator::And,
+                Arc::new(BinaryExpr::new(
+                    Arc::new(BinaryExpr::new(
+                        Arc::new(Column::new("c", 2)),
+                        Operator::LtEq,
+                        Arc::new(Literal::new(ScalarValue::Float32(Some(1075.0)))),
+                    )),
+                    Operator::And,
+                    Arc::new(BinaryExpr::new(
+                        Arc::new(Column::new("a", 0)),
+                        Operator::Gt,
+                        Arc::new(Column::new("b", 1)),
+                    )),
+                )),
+            )),
+        ));
+        let filter: Arc<dyn ExecutionPlan> =
+            Arc::new(FilterExec::try_new(predicate, input)?);
+        let statistics = filter.statistics();
+        // 0.5 (from a) * 0.333333... (from b) * 0.798387... (from c) ≈ 0.1330...
+        // num_rows after ceil => 133.0... => 134
+        // total_byte_size after ceil => 532.0... => 533
+        assert_eq!(statistics.num_rows, Some(134));
+        assert_eq!(statistics.total_byte_size, Some(533));
+        let exp_col_stats = Some(vec![
+            ColumnStatistics {
+                min_value: Some(ScalarValue::Int32(Some(4))),
+                max_value: Some(ScalarValue::Int32(Some(53))),
+                ..Default::default()
+            },
+            ColumnStatistics {
+                min_value: Some(ScalarValue::Int32(Some(3))),
+                max_value: Some(ScalarValue::Int32(Some(3))),
+                ..Default::default()
+            },
+            ColumnStatistics {
+                min_value: Some(ScalarValue::Float32(Some(1000.0))),
+                max_value: Some(ScalarValue::Float32(Some(1075.0))),
+                ..Default::default()
+            },
+        ]);
+        let _ = exp_col_stats
+            .unwrap()
+            .into_iter()
+            .zip(statistics.column_statistics.unwrap())
+            .map(|(expected, actual)| {
+                if actual
+                    .min_value
+                    .clone()
+                    .unwrap()
+                    .get_datatype()
+                    .is_floating()
+                {
+                    // Windows rounds arithmetic operation results differently for floating point numbers.
+                    // Therefore, we check if the actual values are in an epsilon range.
+                    let actual_min = actual.min_value.unwrap();
+                    let actual_max = actual.max_value.unwrap();
+                    let expected_min = expected.min_value.unwrap();
+                    let expected_max = expected.max_value.unwrap();
+                    let eps = ScalarValue::Float32(Some(1e-6));
+
+                    assert!(actual_min.sub(&expected_min).unwrap() < eps);
+                    assert!(actual_min.sub(&expected_min).unwrap() < eps);
+
+                    assert!(actual_max.sub(&expected_max).unwrap() < eps);
+                    assert!(actual_max.sub(&expected_max).unwrap() < eps);
+                } else {
+                    assert_eq!(actual, expected);
+                }
+            });
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_filter_statistics_full_selective() -> Result<()> {
+        // Table:
+        //      a: min=1, max=100
+        //      b: min=1, max=3
+        let schema = Schema::new(vec![
+            Field::new("a", DataType::Int32, false),
+            Field::new("b", DataType::Int32, false),
+        ]);
+        let input = Arc::new(StatisticsExec::new(
+            Statistics {
+                num_rows: Some(1000),
+                total_byte_size: Some(4000),
+                column_statistics: Some(vec![
+                    ColumnStatistics {
+                        min_value: Some(ScalarValue::Int32(Some(1))),
+                        max_value: Some(ScalarValue::Int32(Some(100))),
+                        ..Default::default()
+                    },
+                    ColumnStatistics {
+                        min_value: Some(ScalarValue::Int32(Some(1))),
+                        max_value: Some(ScalarValue::Int32(Some(3))),
+                        ..Default::default()
+                    },
+                ]),
+                ..Default::default()
+            },
+            schema,
+        ));
+        // WHERE a<200 AND 1<=b
+        let predicate = Arc::new(BinaryExpr::new(
+            Arc::new(BinaryExpr::new(
+                Arc::new(Column::new("a", 0)),
+                Operator::Lt,
+                Arc::new(Literal::new(ScalarValue::Int32(Some(200)))),
+            )),
+            Operator::And,
+            Arc::new(BinaryExpr::new(
+                Arc::new(Literal::new(ScalarValue::Int32(Some(1)))),
+                Operator::LtEq,
+                Arc::new(Column::new("b", 1)),
+            )),
+        ));
+        // Since filter predicate passes all entries, statistics after filter shouldn't change.
+        let expected = input.statistics().column_statistics;
+        let filter: Arc<dyn ExecutionPlan> =
+            Arc::new(FilterExec::try_new(predicate, input)?);
+        let statistics = filter.statistics();
+
+        assert_eq!(statistics.num_rows, Some(1000));
+        assert_eq!(statistics.total_byte_size, Some(4000));
+        assert_eq!(statistics.column_statistics, expected);
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_filter_statistics_zero_selective() -> Result<()> {
+        // Table:
+        //      a: min=1, max=100
+        //      b: min=1, max=3
+        let schema = Schema::new(vec![
+            Field::new("a", DataType::Int32, false),
+            Field::new("b", DataType::Int32, false),
+        ]);
+        let input = Arc::new(StatisticsExec::new(
+            Statistics {
+                num_rows: Some(1000),
+                total_byte_size: Some(4000),
+                column_statistics: Some(vec![
+                    ColumnStatistics {
+                        min_value: Some(ScalarValue::Int32(Some(1))),
+                        max_value: Some(ScalarValue::Int32(Some(100))),
+                        ..Default::default()
+                    },
+                    ColumnStatistics {
+                        min_value: Some(ScalarValue::Int32(Some(1))),
+                        max_value: Some(ScalarValue::Int32(Some(3))),
+                        ..Default::default()
+                    },
+                ]),
+                ..Default::default()
+            },
+            schema,
+        ));
+        // WHERE a>200 AND 1<=b
+        let predicate = Arc::new(BinaryExpr::new(
+            Arc::new(BinaryExpr::new(
+                Arc::new(Column::new("a", 0)),
+                Operator::Gt,
+                Arc::new(Literal::new(ScalarValue::Int32(Some(200)))),
+            )),
+            Operator::And,
+            Arc::new(BinaryExpr::new(
+                Arc::new(Literal::new(ScalarValue::Int32(Some(1)))),
+                Operator::LtEq,
+                Arc::new(Column::new("b", 1)),
+            )),
+        ));
+        let filter: Arc<dyn ExecutionPlan> =
+            Arc::new(FilterExec::try_new(predicate, input)?);
+        let statistics = filter.statistics();
+
+        assert_eq!(statistics.num_rows, Some(0));
+        assert_eq!(statistics.total_byte_size, Some(0));
+        assert_eq!(
+            statistics.column_statistics,
+            Some(vec![
+                ColumnStatistics {
+                    min_value: None,
+                    max_value: None,
+                    ..Default::default()
+                },
+                ColumnStatistics {
+                    min_value: None,
+                    max_value: None,
+                    ..Default::default()
+                },
+            ])
+        );
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_filter_statistics_more_inputs() -> Result<()> {
+        let schema = Schema::new(vec![
+            Field::new("a", DataType::Int32, false),
+            Field::new("b", DataType::Int32, false),
+        ]);
+        let input = Arc::new(StatisticsExec::new(
+            Statistics {
+                num_rows: Some(1000),
+                total_byte_size: Some(4000),
+                column_statistics: Some(vec![
+                    ColumnStatistics {
+                        min_value: Some(ScalarValue::Int32(Some(1))),
+                        max_value: Some(ScalarValue::Int32(Some(100))),
+                        ..Default::default()
+                    },
+                    ColumnStatistics {
+                        min_value: Some(ScalarValue::Int32(Some(1))),
+                        max_value: Some(ScalarValue::Int32(Some(100))),
+                        ..Default::default()
+                    },
+                ]),
+                ..Default::default()
+            },
+            schema,
+        ));
+        // WHERE a<50
+        let predicate = Arc::new(BinaryExpr::new(
+            Arc::new(Column::new("a", 0)),
+            Operator::Lt,
+            Arc::new(Literal::new(ScalarValue::Int32(Some(50)))),
+        ));
+        let filter: Arc<dyn ExecutionPlan> =
+            Arc::new(FilterExec::try_new(predicate, input)?);
+        let statistics = filter.statistics();
+
+        assert_eq!(statistics.num_rows, Some(490));
+        assert_eq!(statistics.total_byte_size, Some(1960));
+        assert_eq!(
+            statistics.column_statistics,
+            Some(vec![
+                ColumnStatistics {
+                    min_value: Some(ScalarValue::Int32(Some(1))),
+                    max_value: Some(ScalarValue::Int32(Some(49))),
+                    ..Default::default()
+                },
+                ColumnStatistics {
+                    min_value: Some(ScalarValue::Int32(Some(1))),
+                    max_value: Some(ScalarValue::Int32(Some(100))),
+                    ..Default::default()
+                },
+            ])
+        );
+
+        Ok(())
+    }
 }
