@@ -19,8 +19,8 @@ use async_trait::async_trait;
 use datafusion::execution::context::SessionState;
 use datafusion::{
     catalog::{
-        catalog::{CatalogProvider, MemoryCatalogProvider},
         schema::{MemorySchemaProvider, SchemaProvider},
+        CatalogProvider, MemoryCatalogProvider,
     },
     datasource::{TableProvider, TableType},
 };
@@ -160,12 +160,12 @@ fn table_with_many_types() -> Arc<dyn TableProvider> {
     let batch = RecordBatch::try_new(
         Arc::new(schema.clone()),
         vec![
-            Arc::new(Int32Array::from_slice([1])),
-            Arc::new(Float64Array::from_slice([1.0])),
+            Arc::new(Int32Array::from(vec![1])),
+            Arc::new(Float64Array::from(vec![1.0])),
             Arc::new(StringArray::from(vec![Some("foo")])),
             Arc::new(LargeStringArray::from(vec![Some("bar")])),
-            Arc::new(BinaryArray::from_slice([b"foo" as &[u8]])),
-            Arc::new(LargeBinaryArray::from_slice([b"foo" as &[u8]])),
+            Arc::new(BinaryArray::from(vec![b"foo" as &[u8]])),
+            Arc::new(LargeBinaryArray::from(vec![b"foo" as &[u8]])),
             Arc::new(TimestampNanosecondArray::from(vec![Some(123)])),
         ],
     )
