@@ -172,58 +172,79 @@ impl FromStr for AggregateFunction {
 
 /// Returns the datatype of the aggregate function.
 /// This is used to get the returned data type for aggregate expr.
+#[deprecated(
+    since = "27.0.0",
+    note = "please use `AggregateFunction::return_type` instead"
+)]
 pub fn return_type(
     fun: &AggregateFunction,
     input_expr_types: &[DataType],
 ) -> Result<DataType> {
-    // Note that this function *must* return the same type that the respective physical expression returns
-    // or the execution panics.
+    fun.return_type(input_expr_types)
+}
 
-    let coerced_data_types = crate::type_coercion::aggregates::coerce_types(
-        fun,
-        input_expr_types,
-        &signature(fun),
-    )?;
+impl AggregateFunction {
+    /// Returns the datatype of the aggregate function given its argument types
+    ///
+    /// This is used to get the returned data type for aggregate expr.
+    pub fn return_type(&self, input_expr_types: &[DataType]) -> Result<DataType> {
+        // Note that this function *must* return the same type that the respective physical expression returns
+        // or the execution panics.
 
-    match fun {
-        AggregateFunction::Count | AggregateFunction::ApproxDistinct => {
-            Ok(DataType::Int64)
-        }
-        AggregateFunction::Max | AggregateFunction::Min => {
-            // For min and max agg function, the returned type is same as input type.
-            // The coerced_data_types is same with input_types.
-            Ok(coerced_data_types[0].clone())
-        }
-        AggregateFunction::Sum => sum_return_type(&coerced_data_types[0]),
-        AggregateFunction::BitAnd
-        | AggregateFunction::BitOr
-        | AggregateFunction::BitXor => Ok(coerced_data_types[0].clone()),
-        AggregateFunction::BoolAnd | AggregateFunction::BoolOr => Ok(DataType::Boolean),
-        AggregateFunction::Variance => variance_return_type(&coerced_data_types[0]),
-        AggregateFunction::VariancePop => variance_return_type(&coerced_data_types[0]),
-        AggregateFunction::Covariance => covariance_return_type(&coerced_data_types[0]),
-        AggregateFunction::CovariancePop => {
-            covariance_return_type(&coerced_data_types[0])
-        }
-        AggregateFunction::Correlation => correlation_return_type(&coerced_data_types[0]),
-        AggregateFunction::Stddev => stddev_return_type(&coerced_data_types[0]),
-        AggregateFunction::StddevPop => stddev_return_type(&coerced_data_types[0]),
-        AggregateFunction::Avg => avg_return_type(&coerced_data_types[0]),
-        AggregateFunction::ArrayAgg => Ok(DataType::List(Arc::new(Field::new(
-            "item",
-            coerced_data_types[0].clone(),
-            true,
-        )))),
-        AggregateFunction::ApproxPercentileCont => Ok(coerced_data_types[0].clone()),
-        AggregateFunction::ApproxPercentileContWithWeight => {
-            Ok(coerced_data_types[0].clone())
-        }
-        AggregateFunction::ApproxMedian | AggregateFunction::Median => {
-            Ok(coerced_data_types[0].clone())
-        }
-        AggregateFunction::Grouping => Ok(DataType::Int32),
-        AggregateFunction::FirstValue | AggregateFunction::LastValue => {
-            Ok(coerced_data_types[0].clone())
+        let coerced_data_types = crate::type_coercion::aggregates::coerce_types(
+            self,
+            input_expr_types,
+            &self.signature(),
+        )?;
+
+        match self {
+            AggregateFunction::Count | AggregateFunction::ApproxDistinct => {
+                Ok(DataType::Int64)
+            }
+            AggregateFunction::Max | AggregateFunction::Min => {
+                // For min and max agg function, the returned type is same as input type.
+                // The coerced_data_types is same with input_types.
+                Ok(coerced_data_types[0].clone())
+            }
+            AggregateFunction::Sum => sum_return_type(&coerced_data_types[0]),
+            AggregateFunction::BitAnd
+            | AggregateFunction::BitOr
+            | AggregateFunction::BitXor => Ok(coerced_data_types[0].clone()),
+            AggregateFunction::BoolAnd | AggregateFunction::BoolOr => {
+                Ok(DataType::Boolean)
+            }
+            AggregateFunction::Variance => variance_return_type(&coerced_data_types[0]),
+            AggregateFunction::VariancePop => {
+                variance_return_type(&coerced_data_types[0])
+            }
+            AggregateFunction::Covariance => {
+                covariance_return_type(&coerced_data_types[0])
+            }
+            AggregateFunction::CovariancePop => {
+                covariance_return_type(&coerced_data_types[0])
+            }
+            AggregateFunction::Correlation => {
+                correlation_return_type(&coerced_data_types[0])
+            }
+            AggregateFunction::Stddev => stddev_return_type(&coerced_data_types[0]),
+            AggregateFunction::StddevPop => stddev_return_type(&coerced_data_types[0]),
+            AggregateFunction::Avg => avg_return_type(&coerced_data_types[0]),
+            AggregateFunction::ArrayAgg => Ok(DataType::List(Arc::new(Field::new(
+                "item",
+                coerced_data_types[0].clone(),
+                true,
+            )))),
+            AggregateFunction::ApproxPercentileCont => Ok(coerced_data_types[0].clone()),
+            AggregateFunction::ApproxPercentileContWithWeight => {
+                Ok(coerced_data_types[0].clone())
+            }
+            AggregateFunction::ApproxMedian | AggregateFunction::Median => {
+                Ok(coerced_data_types[0].clone())
+            }
+            AggregateFunction::Grouping => Ok(DataType::Int32),
+            AggregateFunction::FirstValue | AggregateFunction::LastValue => {
+                Ok(coerced_data_types[0].clone())
+            }
         }
     }
 }
@@ -236,79 +257,94 @@ pub fn sum_type_of_avg(input_expr_types: &[DataType]) -> Result<DataType> {
     let coerced_data_types = crate::type_coercion::aggregates::coerce_types(
         &fun,
         input_expr_types,
-        &signature(&fun),
+        &fun.signature(),
     )?;
     avg_sum_type(&coerced_data_types[0])
 }
 
 /// the signatures supported by the function `fun`.
+#[deprecated(
+    since = "27.0.0",
+    note = "please use `AggregateFunction::signature` instead"
+)]
 pub fn signature(fun: &AggregateFunction) -> Signature {
-    // note: the physical expression must accept the type returned by this function or the execution panics.
-    match fun {
-        AggregateFunction::Count => Signature::variadic_any(Volatility::Immutable),
-        AggregateFunction::ApproxDistinct
-        | AggregateFunction::Grouping
-        | AggregateFunction::ArrayAgg => Signature::any(1, Volatility::Immutable),
-        AggregateFunction::Min | AggregateFunction::Max => {
-            let valid = STRINGS
-                .iter()
-                .chain(NUMERICS.iter())
-                .chain(TIMESTAMPS.iter())
-                .chain(DATES.iter())
-                .chain(TIMES.iter())
-                .cloned()
-                .collect::<Vec<_>>();
-            Signature::uniform(1, valid, Volatility::Immutable)
-        }
-        AggregateFunction::BitAnd
-        | AggregateFunction::BitOr
-        | AggregateFunction::BitXor => {
-            Signature::uniform(1, INTEGERS.to_vec(), Volatility::Immutable)
-        }
-        AggregateFunction::BoolAnd | AggregateFunction::BoolOr => {
-            Signature::uniform(1, vec![DataType::Boolean], Volatility::Immutable)
-        }
-        AggregateFunction::Avg
-        | AggregateFunction::Sum
-        | AggregateFunction::Variance
-        | AggregateFunction::VariancePop
-        | AggregateFunction::Stddev
-        | AggregateFunction::StddevPop
-        | AggregateFunction::Median
-        | AggregateFunction::ApproxMedian
-        | AggregateFunction::FirstValue
-        | AggregateFunction::LastValue => {
-            Signature::uniform(1, NUMERICS.to_vec(), Volatility::Immutable)
-        }
-        AggregateFunction::Covariance | AggregateFunction::CovariancePop => {
-            Signature::uniform(2, NUMERICS.to_vec(), Volatility::Immutable)
-        }
-        AggregateFunction::Correlation => {
-            Signature::uniform(2, NUMERICS.to_vec(), Volatility::Immutable)
-        }
-        AggregateFunction::ApproxPercentileCont => {
-            // Accept any numeric value paired with a float64 percentile
-            let with_tdigest_size = NUMERICS.iter().map(|t| {
-                TypeSignature::Exact(vec![t.clone(), DataType::Float64, t.clone()])
-            });
-            Signature::one_of(
+    fun.signature()
+}
+
+impl AggregateFunction {
+    /// the signatures supported by the function `fun`.
+    pub fn signature(&self) -> Signature {
+        // note: the physical expression must accept the type returned by this function or the execution panics.
+        match self {
+            AggregateFunction::Count => Signature::variadic_any(Volatility::Immutable),
+            AggregateFunction::ApproxDistinct
+            | AggregateFunction::Grouping
+            | AggregateFunction::ArrayAgg => Signature::any(1, Volatility::Immutable),
+            AggregateFunction::Min | AggregateFunction::Max => {
+                let valid = STRINGS
+                    .iter()
+                    .chain(NUMERICS.iter())
+                    .chain(TIMESTAMPS.iter())
+                    .chain(DATES.iter())
+                    .chain(TIMES.iter())
+                    .cloned()
+                    .collect::<Vec<_>>();
+                Signature::uniform(1, valid, Volatility::Immutable)
+            }
+            AggregateFunction::BitAnd
+            | AggregateFunction::BitOr
+            | AggregateFunction::BitXor => {
+                Signature::uniform(1, INTEGERS.to_vec(), Volatility::Immutable)
+            }
+            AggregateFunction::BoolAnd | AggregateFunction::BoolOr => {
+                Signature::uniform(1, vec![DataType::Boolean], Volatility::Immutable)
+            }
+            AggregateFunction::Avg
+            | AggregateFunction::Sum
+            | AggregateFunction::Variance
+            | AggregateFunction::VariancePop
+            | AggregateFunction::Stddev
+            | AggregateFunction::StddevPop
+            | AggregateFunction::Median
+            | AggregateFunction::ApproxMedian
+            | AggregateFunction::FirstValue
+            | AggregateFunction::LastValue => {
+                Signature::uniform(1, NUMERICS.to_vec(), Volatility::Immutable)
+            }
+            AggregateFunction::Covariance | AggregateFunction::CovariancePop => {
+                Signature::uniform(2, NUMERICS.to_vec(), Volatility::Immutable)
+            }
+            AggregateFunction::Correlation => {
+                Signature::uniform(2, NUMERICS.to_vec(), Volatility::Immutable)
+            }
+            AggregateFunction::ApproxPercentileCont => {
+                // Accept any numeric value paired with a float64 percentile
+                let with_tdigest_size = NUMERICS.iter().map(|t| {
+                    TypeSignature::Exact(vec![t.clone(), DataType::Float64, t.clone()])
+                });
+                Signature::one_of(
+                    NUMERICS
+                        .iter()
+                        .map(|t| TypeSignature::Exact(vec![t.clone(), DataType::Float64]))
+                        .chain(with_tdigest_size)
+                        .collect(),
+                    Volatility::Immutable,
+                )
+            }
+            AggregateFunction::ApproxPercentileContWithWeight => Signature::one_of(
+                // Accept any numeric value paired with a float64 percentile
                 NUMERICS
                     .iter()
-                    .map(|t| TypeSignature::Exact(vec![t.clone(), DataType::Float64]))
-                    .chain(with_tdigest_size)
+                    .map(|t| {
+                        TypeSignature::Exact(vec![
+                            t.clone(),
+                            t.clone(),
+                            DataType::Float64,
+                        ])
+                    })
                     .collect(),
                 Volatility::Immutable,
-            )
+            ),
         }
-        AggregateFunction::ApproxPercentileContWithWeight => Signature::one_of(
-            // Accept any numeric value paired with a float64 percentile
-            NUMERICS
-                .iter()
-                .map(|t| {
-                    TypeSignature::Exact(vec![t.clone(), t.clone(), DataType::Float64])
-                })
-                .collect(),
-            Volatility::Immutable,
-        ),
     }
 }
