@@ -82,7 +82,7 @@ use std::sync::Arc;
 #[derive(Clone, PartialEq, Eq, Hash, Debug)]
 pub enum Expr {
     /// An expression with a specific name.
-    Alias(Box<Expr>, String),
+    Alias(Alias),
     /// A named reference to a qualified filed in a schema.
     Column(Column),
     /// A named reference to a variable in a registry.
@@ -93,8 +93,6 @@ pub enum Expr {
     BinaryExpr(BinaryExpr),
     /// LIKE expression
     Like(Like),
-    /// Case-insensitive LIKE expression
-    ILike(Like),
     /// LIKE expression that uses regular expressions
     SimilarTo(Like),
     /// Negation of an expression. The expression's type must be a boolean to make sense.
@@ -180,6 +178,22 @@ pub enum Expr {
     OuterReferenceColumn(DataType, Column),
 }
 
+/// Alias expression
+#[derive(Clone, PartialEq, Eq, Hash, Debug)]
+pub struct Alias {
+    pub expr: Box<Expr>,
+    pub name: String,
+}
+
+impl Alias {
+    pub fn new(expr: Expr, name: impl Into<String>) -> Self {
+        Self {
+            expr: Box::new(expr),
+            name: name.into(),
+        }
+    }
+}
+
 /// Binary expression
 #[derive(Clone, PartialEq, Eq, Hash, Debug)]
 pub struct BinaryExpr {
@@ -264,6 +278,8 @@ pub struct Like {
     pub expr: Box<Expr>,
     pub pattern: Box<Expr>,
     pub escape_char: Option<char>,
+    /// Whether to ignore case on comparing
+    pub case_insensitive: bool,
 }
 
 impl Like {
@@ -273,12 +289,14 @@ impl Like {
         expr: Box<Expr>,
         pattern: Box<Expr>,
         escape_char: Option<char>,
+        case_insensitive: bool,
     ) -> Self {
         Self {
             negated,
             expr,
             pattern,
             escape_char,
+            case_insensitive,
         }
     }
 }
@@ -675,7 +693,6 @@ impl Expr {
             Expr::IsNotNull(..) => "IsNotNull",
             Expr::IsNull(..) => "IsNull",
             Expr::Like { .. } => "Like",
-            Expr::ILike { .. } => "ILike",
             Expr::SimilarTo { .. } => "RLike",
             Expr::IsTrue(..) => "IsTrue",
             Expr::IsFalse(..) => "IsFalse",
@@ -741,22 +758,40 @@ impl Expr {
 
     /// Return `self LIKE other`
     pub fn like(self, other: Expr) -> Expr {
-        Expr::Like(Like::new(false, Box::new(self), Box::new(other), None))
+        Expr::Like(Like::new(
+            false,
+            Box::new(self),
+            Box::new(other),
+            None,
+            false,
+        ))
     }
 
     /// Return `self NOT LIKE other`
     pub fn not_like(self, other: Expr) -> Expr {
-        Expr::Like(Like::new(true, Box::new(self), Box::new(other), None))
+        Expr::Like(Like::new(
+            true,
+            Box::new(self),
+            Box::new(other),
+            None,
+            false,
+        ))
     }
 
     /// Return `self ILIKE other`
     pub fn ilike(self, other: Expr) -> Expr {
-        Expr::ILike(Like::new(false, Box::new(self), Box::new(other), None))
+        Expr::Like(Like::new(
+            false,
+            Box::new(self),
+            Box::new(other),
+            None,
+            true,
+        ))
     }
 
     /// Return `self NOT ILIKE other`
     pub fn not_ilike(self, other: Expr) -> Expr {
-        Expr::ILike(Like::new(true, Box::new(self), Box::new(other), None))
+        Expr::Like(Like::new(true, Box::new(self), Box::new(other), None, true))
     }
 
     /// Return the name to use for the specific Expr, recursing into
@@ -789,14 +824,14 @@ impl Expr {
                 asc,
                 nulls_first,
             }) => Expr::Sort(Sort::new(Box::new(expr.alias(name)), asc, nulls_first)),
-            _ => Expr::Alias(Box::new(self), name.into()),
+            _ => Expr::Alias(Alias::new(self, name.into())),
         }
     }
 
     /// Remove an alias from an expression if one exists.
     pub fn unalias(self) -> Expr {
         match self {
-            Expr::Alias(expr, _) => expr.as_ref().clone(),
+            Expr::Alias(alias) => alias.expr.as_ref().clone(),
             _ => self,
         }
     }
@@ -914,7 +949,7 @@ macro_rules! expr_vec_fmt {
 impl fmt::Display for Expr {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
-            Expr::Alias(expr, alias) => write!(f, "{expr} AS {alias}"),
+            Expr::Alias(Alias { expr, name, .. }) => write!(f, "{expr} AS {name}"),
             Expr::Column(c) => write!(f, "{c}"),
             Expr::OuterReferenceColumn(_, c) => write!(f, "outer_ref({c})"),
             Expr::ScalarVariable(_, var_names) => write!(f, "{}", var_names.join(".")),
@@ -1061,31 +1096,17 @@ impl fmt::Display for Expr {
                 expr,
                 pattern,
                 escape_char,
+                case_insensitive,
             }) => {
                 write!(f, "{expr}")?;
+                let op_name = if *case_insensitive { "ILIKE" } else { "LIKE" };
                 if *negated {
                     write!(f, " NOT")?;
                 }
                 if let Some(char) = escape_char {
-                    write!(f, " LIKE {pattern} ESCAPE '{char}'")
+                    write!(f, " {op_name} {pattern} ESCAPE '{char}'")
                 } else {
-                    write!(f, " LIKE {pattern}")
-                }
-            }
-            Expr::ILike(Like {
-                negated,
-                expr,
-                pattern,
-                escape_char,
-            }) => {
-                write!(f, "{expr}")?;
-                if *negated {
-                    write!(f, " NOT")?;
-                }
-                if let Some(char) = escape_char {
-                    write!(f, " ILIKE {pattern} ESCAPE '{char}'")
-                } else {
-                    write!(f, " ILIKE {pattern}")
+                    write!(f, " {op_name} {pattern}")
                 }
             }
             Expr::SimilarTo(Like {
@@ -1093,6 +1114,7 @@ impl fmt::Display for Expr {
                 expr,
                 pattern,
                 escape_char,
+                case_insensitive: _,
             }) => {
                 write!(f, "{expr}")?;
                 if *negated {
@@ -1180,7 +1202,7 @@ fn create_function_name(fun: &str, distinct: bool, args: &[Expr]) -> Result<Stri
 /// This function recursively transverses the expression for names such as "CAST(a > 2)".
 fn create_name(e: &Expr) -> Result<String> {
     match e {
-        Expr::Alias(_, name) => Ok(name.clone()),
+        Expr::Alias(Alias { name, .. }) => Ok(name.clone()),
         Expr::Column(c) => Ok(c.flat_name()),
         Expr::OuterReferenceColumn(_, c) => Ok(format!("outer_ref({})", c.flat_name())),
         Expr::ScalarVariable(_, variable_names) => Ok(variable_names.join(".")),
@@ -1195,30 +1217,13 @@ fn create_name(e: &Expr) -> Result<String> {
             expr,
             pattern,
             escape_char,
+            case_insensitive,
         }) => {
             let s = format!(
-                "{} {} {} {}",
+                "{} {}{} {} {}",
                 expr,
-                if *negated { "NOT LIKE" } else { "LIKE" },
-                pattern,
-                if let Some(char) = escape_char {
-                    format!("CHAR '{char}'")
-                } else {
-                    "".to_string()
-                }
-            );
-            Ok(s)
-        }
-        Expr::ILike(Like {
-            negated,
-            expr,
-            pattern,
-            escape_char,
-        }) => {
-            let s = format!(
-                "{} {} {} {}",
-                expr,
-                if *negated { "NOT ILIKE" } else { "ILIKE" },
+                if *negated { "NOT " } else { "" },
+                if *case_insensitive { "ILIKE" } else { "LIKE" },
                 pattern,
                 if let Some(char) = escape_char {
                     format!("CHAR '{char}'")
@@ -1233,6 +1238,7 @@ fn create_name(e: &Expr) -> Result<String> {
             expr,
             pattern,
             escape_char,
+            case_insensitive: _,
         }) => {
             let s = format!(
                 "{} {} {} {}",
