@@ -467,7 +467,7 @@ pub fn is_sum_support_arg_type(arg_type: &DataType) -> bool {
         _ => matches!(
             arg_type,
             arg_type if NUMERICS.contains(arg_type)
-            || matches!(arg_type, DataType::Decimal128(_, _))
+            || matches!(arg_type, DataType::Decimal128(_, _) | DataType::Decimal256(_, _))
         ),
     }
 }
@@ -480,7 +480,7 @@ pub fn is_avg_support_arg_type(arg_type: &DataType) -> bool {
         _ => matches!(
             arg_type,
             arg_type if NUMERICS.contains(arg_type)
-                || matches!(arg_type, DataType::Decimal128(_, _))
+                || matches!(arg_type, DataType::Decimal128(_, _)| DataType::Decimal256(_, _))
         ),
     }
 }
@@ -539,7 +539,6 @@ pub fn is_approx_percentile_cont_supported_arg_type(arg_type: &DataType) -> bool
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::aggregate_function;
     use arrow::datatypes::DataType;
 
     #[test]
@@ -547,21 +546,21 @@ mod tests {
         // test input args with error number input types
         let fun = AggregateFunction::Min;
         let input_types = vec![DataType::Int64, DataType::Int32];
-        let signature = aggregate_function::signature(&fun);
+        let signature = fun.signature();
         let result = coerce_types(&fun, &input_types, &signature);
         assert_eq!("Error during planning: The function Min expects 1 arguments, but 2 were provided", result.unwrap_err().to_string());
 
         // test input args is invalid data type for sum or avg
         let fun = AggregateFunction::Sum;
         let input_types = vec![DataType::Utf8];
-        let signature = aggregate_function::signature(&fun);
+        let signature = fun.signature();
         let result = coerce_types(&fun, &input_types, &signature);
         assert_eq!(
             "Error during planning: The function Sum does not support inputs of type Utf8.",
             result.unwrap_err().to_string()
         );
         let fun = AggregateFunction::Avg;
-        let signature = aggregate_function::signature(&fun);
+        let signature = fun.signature();
         let result = coerce_types(&fun, &input_types, &signature);
         assert_eq!(
             "Error during planning: The function Avg does not support inputs of type Utf8.",
@@ -580,11 +579,12 @@ mod tests {
         let input_types = vec![
             vec![DataType::Int32],
             vec![DataType::Decimal128(10, 2)],
+            vec![DataType::Decimal256(1, 1)],
             vec![DataType::Utf8],
         ];
         for fun in funs {
             for input_type in &input_types {
-                let signature = aggregate_function::signature(&fun);
+                let signature = fun.signature();
                 let result = coerce_types(&fun, input_type, &signature);
                 assert_eq!(*input_type, result.unwrap());
             }
@@ -595,10 +595,11 @@ mod tests {
             vec![DataType::Int32],
             vec![DataType::Float32],
             vec![DataType::Decimal128(20, 3)],
+            vec![DataType::Decimal256(20, 3)],
         ];
         for fun in funs {
             for input_type in &input_types {
-                let signature = aggregate_function::signature(&fun);
+                let signature = fun.signature();
                 let result = coerce_types(&fun, input_type, &signature);
                 assert_eq!(*input_type, result.unwrap());
             }
@@ -618,8 +619,7 @@ mod tests {
             vec![DataType::Float64, DataType::Float64],
         ];
         for input_type in &input_types {
-            let signature =
-                aggregate_function::signature(&AggregateFunction::ApproxPercentileCont);
+            let signature = AggregateFunction::ApproxPercentileCont.signature();
             let result = coerce_types(
                 &AggregateFunction::ApproxPercentileCont,
                 input_type,
