@@ -20,7 +20,7 @@
 use crate::{OptimizerConfig, OptimizerRule};
 use datafusion_common::{plan_err, Column, DFSchemaRef};
 use datafusion_common::{DFSchema, Result};
-use datafusion_expr::expr::BinaryExpr;
+use datafusion_expr::expr::{Alias, BinaryExpr};
 use datafusion_expr::expr_rewriter::{replace_col, strip_outer_reference};
 use datafusion_expr::{
     and,
@@ -74,7 +74,7 @@ fn split_conjunction_impl<'a>(expr: &'a Expr, mut exprs: Vec<&'a Expr>) -> Vec<&
             let exprs = split_conjunction_impl(left, exprs);
             split_conjunction_impl(right, exprs)
         }
-        Expr::Alias(expr, _) => split_conjunction_impl(expr, exprs),
+        Expr::Alias(Alias { expr, .. }) => split_conjunction_impl(expr, exprs),
         other => {
             exprs.push(other);
             exprs
@@ -143,7 +143,9 @@ fn split_binary_owned_impl(
             let exprs = split_binary_owned_impl(*left, operator, exprs);
             split_binary_owned_impl(*right, operator, exprs)
         }
-        Expr::Alias(expr, _) => split_binary_owned_impl(*expr, operator, exprs),
+        Expr::Alias(Alias { expr, .. }) => {
+            split_binary_owned_impl(*expr, operator, exprs)
+        }
         other => {
             exprs.push(other);
             exprs
@@ -168,7 +170,7 @@ fn split_binary_impl<'a>(
             let exprs = split_binary_impl(left, operator, exprs);
             split_binary_impl(right, operator, exprs)
         }
-        Expr::Alias(expr, _) => split_binary_impl(expr, operator, exprs),
+        Expr::Alias(Alias { expr, .. }) => split_binary_impl(expr, operator, exprs),
         other => {
             exprs.push(other);
             exprs
