@@ -15,15 +15,18 @@
 // specific language governing permissions and limitations
 // under the License.
 
-//! Defines physical expressions that can evaluated at runtime during query execution
+//! Defines BitAnd, BitOr, and BitXor Aggregate accumulators
 
 use ahash::RandomState;
 use std::any::Any;
 use std::convert::TryFrom;
 use std::sync::Arc;
 
-use crate::{AggregateExpr, PhysicalExpr};
-use arrow::datatypes::DataType;
+use crate::{AggregateExpr, GroupsAccumulator, PhysicalExpr};
+use arrow::datatypes::{
+    DataType, Int16Type, Int32Type, Int64Type, Int8Type, UInt16Type, UInt32Type,
+    UInt64Type, UInt8Type,
+};
 use arrow::{
     array::{
         ArrayRef, Int16Array, Int32Array, Int64Array, Int8Array, UInt16Array,
@@ -35,14 +38,25 @@ use datafusion_common::{downcast_value, DataFusionError, Result, ScalarValue};
 use datafusion_expr::Accumulator;
 use std::collections::HashSet;
 
-use crate::aggregate::row_accumulator::{
-    is_row_accumulator_support_dtype, RowAccumulator,
-};
+use crate::aggregate::groups_accumulator::prim_op::PrimitiveGroupsAccumulator;
 use crate::aggregate::utils::down_cast_any_ref;
 use crate::expressions::format_state_name;
 use arrow::array::Array;
 use arrow::compute::{bit_and, bit_or, bit_xor};
-use datafusion_row::accessor::RowAccessor;
+
+/// Creates a [`PrimitiveGroupsAccumulator`] with the specified
+/// [`ArrowPrimitiveType`] that initailizes each accumulator to $START
+/// and applies `$FN` to each element
+///
+/// [`ArrowPrimitiveType`]: arrow::datatypes::ArrowPrimitiveType
+macro_rules! instantiate_accumulator {
+    ($SELF:expr, $START:expr, $PRIMTYPE:ident, $FN:expr) => {{
+        Ok(Box::new(
+            PrimitiveGroupsAccumulator::<$PRIMTYPE, _>::new(&$SELF.data_type, $FN)
+                .with_starting_value($START),
+        ))
+    }};
+}
 
 // returns the new value after bit_and/bit_or/bit_xor with the new values, taking nullability into account
 macro_rules! typed_bit_and_or_xor_batch {
@@ -105,82 +119,6 @@ fn bit_xor_batch(values: &ArrayRef) -> Result<ScalarValue> {
     bit_and_or_xor_batch!(values, bit_xor)
 }
 
-// bit_and/bit_or/bit_xor of two scalar values.
-macro_rules! typed_bit_and_or_xor_v2 {
-    ($INDEX:ident, $ACC:ident, $SCALAR:expr, $TYPE:ident, $OP:ident) => {{
-        paste::item! {
-            match $SCALAR {
-                None => {}
-                Some(v) => $ACC.[<$OP _ $TYPE>]($INDEX, *v as $TYPE)
-            }
-        }
-    }};
-}
-
-macro_rules! bit_and_or_xor_v2 {
-    ($INDEX:ident, $ACC:ident, $SCALAR:expr, $OP:ident) => {{
-        Ok(match $SCALAR {
-            ScalarValue::UInt64(rhs) => {
-                typed_bit_and_or_xor_v2!($INDEX, $ACC, rhs, u64, $OP)
-            }
-            ScalarValue::UInt32(rhs) => {
-                typed_bit_and_or_xor_v2!($INDEX, $ACC, rhs, u32, $OP)
-            }
-            ScalarValue::UInt16(rhs) => {
-                typed_bit_and_or_xor_v2!($INDEX, $ACC, rhs, u16, $OP)
-            }
-            ScalarValue::UInt8(rhs) => {
-                typed_bit_and_or_xor_v2!($INDEX, $ACC, rhs, u8, $OP)
-            }
-            ScalarValue::Int64(rhs) => {
-                typed_bit_and_or_xor_v2!($INDEX, $ACC, rhs, i64, $OP)
-            }
-            ScalarValue::Int32(rhs) => {
-                typed_bit_and_or_xor_v2!($INDEX, $ACC, rhs, i32, $OP)
-            }
-            ScalarValue::Int16(rhs) => {
-                typed_bit_and_or_xor_v2!($INDEX, $ACC, rhs, i16, $OP)
-            }
-            ScalarValue::Int8(rhs) => {
-                typed_bit_and_or_xor_v2!($INDEX, $ACC, rhs, i8, $OP)
-            }
-            ScalarValue::Null => {
-                // do nothing
-            }
-            e => {
-                return Err(DataFusionError::Internal(format!(
-                    "BIT AND/BIT OR/BIT XOR is not expected to receive scalars of incompatible types {:?}",
-                    e
-                )))
-            }
-        })
-    }};
-}
-
-pub fn bit_and_row(
-    index: usize,
-    accessor: &mut RowAccessor,
-    s: &ScalarValue,
-) -> Result<()> {
-    bit_and_or_xor_v2!(index, accessor, s, bitand)
-}
-
-pub fn bit_or_row(
-    index: usize,
-    accessor: &mut RowAccessor,
-    s: &ScalarValue,
-) -> Result<()> {
-    bit_and_or_xor_v2!(index, accessor, s, bitor)
-}
-
-pub fn bit_xor_row(
-    index: usize,
-    accessor: &mut RowAccessor,
-    s: &ScalarValue,
-) -> Result<()> {
-    bit_and_or_xor_v2!(index, accessor, s, bitxor)
-}
-
 /// BIT_AND aggregate expression
 #[derive(Debug, Clone)]
 pub struct BitAnd {
@@ -240,18 +178,51 @@ impl AggregateExpr for BitAnd {
         &self.name
     }
 
-    fn row_accumulator_supported(&self) -> bool {
-        is_row_accumulator_support_dtype(&self.data_type)
+    fn groups_accumulator_supported(&self) -> bool {
+        true
     }
 
-    fn create_row_accumulator(
-        &self,
-        start_index: usize,
-    ) -> Result<Box<dyn RowAccumulator>> {
-        Ok(Box::new(BitAndRowAccumulator::new(
-            start_index,
-            self.data_type.clone(),
-        )))
+    fn create_groups_accumulator(&self) -> Result<Box<dyn GroupsAccumulator>> {
+        use std::ops::BitAndAssign;
+        // Note the default value for BitAnd should be all set
+        // (e.g. `0b11...111`) use MAX / -1 here to get appropriate
+        // bit pattern for each type
+        match self.data_type {
+            DataType::Int8 => {
+                instantiate_accumulator!(self, -1, Int8Type, |x, y| x.bitand_assign(y))
+            }
+            DataType::Int16 => {
+                instantiate_accumulator!(self, -1, Int16Type, |x, y| x.bitand_assign(y))
+            }
+            DataType::Int32 => {
+                instantiate_accumulator!(self, -1, Int32Type, |x, y| x.bitand_assign(y))
+            }
+            DataType::Int64 => {
+                instantiate_accumulator!(self, -1, Int64Type, |x, y| x.bitand_assign(y))
+            }
+            DataType::UInt8 => {
+                instantiate_accumulator!(self, u8::MAX, UInt8Type, |x, y| x
+                    .bitand_assign(y))
+            }
+            DataType::UInt16 => {
+                instantiate_accumulator!(self, u16::MAX, UInt16Type, |x, y| x
+                    .bitand_assign(y))
+            }
+            DataType::UInt32 => {
+                instantiate_accumulator!(self, u32::MAX, UInt32Type, |x, y| x
+                    .bitand_assign(y))
+            }
+            DataType::UInt64 => {
+                instantiate_accumulator!(self, u64::MAX, UInt64Type, |x, y| x
+                    .bitand_assign(y))
+            }
+
+            _ => Err(DataFusionError::NotImplemented(format!(
+                "GroupsAccumulator not supported for {} with {}",
+                self.name(),
+                self.data_type
+            ))),
+        }
     }
 
     fn reverse_expr(&self) -> Option<Arc<dyn AggregateExpr>> {
@@ -313,64 +284,6 @@ impl Accumulator for BitAndAccumulator {
     }
 }
 
-#[derive(Debug)]
-struct BitAndRowAccumulator {
-    index: usize,
-    datatype: DataType,
-}
-
-impl BitAndRowAccumulator {
-    pub fn new(index: usize, datatype: DataType) -> Self {
-        Self { index, datatype }
-    }
-}
-
-impl RowAccumulator for BitAndRowAccumulator {
-    fn update_batch(
-        &mut self,
-        values: &[ArrayRef],
-        accessor: &mut RowAccessor,
-    ) -> Result<()> {
-        let values = &values[0];
-        let delta = &bit_and_batch(values)?;
-        bit_and_row(self.index, accessor, delta)
-    }
-
-    fn update_scalar_values(
-        &mut self,
-        values: &[ScalarValue],
-        accessor: &mut RowAccessor,
-    ) -> Result<()> {
-        let value = &values[0];
-        bit_and_row(self.index, accessor, value)
-    }
-
-    fn update_scalar(
-        &mut self,
-        value: &ScalarValue,
-        accessor: &mut RowAccessor,
-    ) -> Result<()> {
-        bit_and_row(self.index, accessor, value)
-    }
-
-    fn merge_batch(
-        &mut self,
-        states: &[ArrayRef],
-        accessor: &mut RowAccessor,
-    ) -> Result<()> {
-        self.update_batch(states, accessor)
-    }
-
-    fn evaluate(&self, accessor: &RowAccessor) -> Result<ScalarValue> {
-        Ok(accessor.get_as_scalar(&self.datatype, self.index))
-    }
-
-    #[inline(always)]
-    fn state_index(&self) -> usize {
-        self.index
-    }
-}
-
 /// BIT_OR aggregate expression
 #[derive(Debug, Clone)]
 pub struct BitOr {
@@ -430,18 +343,44 @@ impl AggregateExpr for BitOr {
         &self.name
     }
 
-    fn row_accumulator_supported(&self) -> bool {
-        is_row_accumulator_support_dtype(&self.data_type)
+    fn groups_accumulator_supported(&self) -> bool {
+        true
     }
 
-    fn create_row_accumulator(
-        &self,
-        start_index: usize,
-    ) -> Result<Box<dyn RowAccumulator>> {
-        Ok(Box::new(BitOrRowAccumulator::new(
-            start_index,
-            self.data_type.clone(),
-        )))
+    fn create_groups_accumulator(&self) -> Result<Box<dyn GroupsAccumulator>> {
+        use std::ops::BitOrAssign;
+        match self.data_type {
+            DataType::Int8 => {
+                instantiate_accumulator!(self, 0, Int8Type, |x, y| x.bitor_assign(y))
+            }
+            DataType::Int16 => {
+                instantiate_accumulator!(self, 0, Int16Type, |x, y| x.bitor_assign(y))
+            }
+            DataType::Int32 => {
+                instantiate_accumulator!(self, 0, Int32Type, |x, y| x.bitor_assign(y))
+            }
+            DataType::Int64 => {
+                instantiate_accumulator!(self, 0, Int64Type, |x, y| x.bitor_assign(y))
+            }
+            DataType::UInt8 => {
+                instantiate_accumulator!(self, 0, UInt8Type, |x, y| x.bitor_assign(y))
+            }
+            DataType::UInt16 => {
+                instantiate_accumulator!(self, 0, UInt16Type, |x, y| x.bitor_assign(y))
+            }
+            DataType::UInt32 => {
+                instantiate_accumulator!(self, 0, UInt32Type, |x, y| x.bitor_assign(y))
+            }
+            DataType::UInt64 => {
+                instantiate_accumulator!(self, 0, UInt64Type, |x, y| x.bitor_assign(y))
+            }
+
+            _ => Err(DataFusionError::NotImplemented(format!(
+                "GroupsAccumulator not supported for {} with {}",
+                self.name(),
+                self.data_type
+            ))),
+        }
     }
 
     fn reverse_expr(&self) -> Option<Arc<dyn AggregateExpr>> {
@@ -503,65 +442,6 @@ impl Accumulator for BitOrAccumulator {
     }
 }
 
-#[derive(Debug)]
-struct BitOrRowAccumulator {
-    index: usize,
-    datatype: DataType,
-}
-
-impl BitOrRowAccumulator {
-    pub fn new(index: usize, datatype: DataType) -> Self {
-        Self { index, datatype }
-    }
-}
-
-impl RowAccumulator for BitOrRowAccumulator {
-    fn update_batch(
-        &mut self,
-        values: &[ArrayRef],
-        accessor: &mut RowAccessor,
-    ) -> Result<()> {
-        let values = &values[0];
-        let delta = &bit_or_batch(values)?;
-        bit_or_row(self.index, accessor, delta)?;
-        Ok(())
-    }
-
-    fn update_scalar_values(
-        &mut self,
-        values: &[ScalarValue],
-        accessor: &mut RowAccessor,
-    ) -> Result<()> {
-        let value = &values[0];
-        bit_or_row(self.index, accessor, value)
-    }
-
-    fn update_scalar(
-        &mut self,
-        value: &ScalarValue,
-        accessor: &mut RowAccessor,
-    ) -> Result<()> {
-        bit_or_row(self.index, accessor, value)
-    }
-
-    fn merge_batch(
-        &mut self,
-        states: &[ArrayRef],
-        accessor: &mut RowAccessor,
-    ) -> Result<()> {
-        self.update_batch(states, accessor)
-    }
-
-    fn evaluate(&self, accessor: &RowAccessor) -> Result<ScalarValue> {
-        Ok(accessor.get_as_scalar(&self.datatype, self.index))
-    }
-
-    #[inline(always)]
-    fn state_index(&self) -> usize {
-        self.index
-    }
-}
-
 /// BIT_XOR aggregate expression
 #[derive(Debug, Clone)]
 pub struct BitXor {
@@ -621,18 +501,44 @@ impl AggregateExpr for BitXor {
         &self.name
     }
 
-    fn row_accumulator_supported(&self) -> bool {
-        is_row_accumulator_support_dtype(&self.data_type)
+    fn groups_accumulator_supported(&self) -> bool {
+        true
     }
 
-    fn create_row_accumulator(
-        &self,
-        start_index: usize,
-    ) -> Result<Box<dyn RowAccumulator>> {
-        Ok(Box::new(BitXorRowAccumulator::new(
-            start_index,
-            self.data_type.clone(),
-        )))
+    fn create_groups_accumulator(&self) -> Result<Box<dyn GroupsAccumulator>> {
+        use std::ops::BitXorAssign;
+        match self.data_type {
+            DataType::Int8 => {
+                instantiate_accumulator!(self, 0, Int8Type, |x, y| x.bitxor_assign(y))
+            }
+            DataType::Int16 => {
+                instantiate_accumulator!(self, 0, Int16Type, |x, y| x.bitxor_assign(y))
+            }
+            DataType::Int32 => {
+                instantiate_accumulator!(self, 0, Int32Type, |x, y| x.bitxor_assign(y))
+            }
+            DataType::Int64 => {
+                instantiate_accumulator!(self, 0, Int64Type, |x, y| x.bitxor_assign(y))
+            }
+            DataType::UInt8 => {
+                instantiate_accumulator!(self, 0, UInt8Type, |x, y| x.bitxor_assign(y))
+            }
+            DataType::UInt16 => {
+                instantiate_accumulator!(self, 0, UInt16Type, |x, y| x.bitxor_assign(y))
+            }
+            DataType::UInt32 => {
+                instantiate_accumulator!(self, 0, UInt32Type, |x, y| x.bitxor_assign(y))
+            }
+            DataType::UInt64 => {
+                instantiate_accumulator!(self, 0, UInt64Type, |x, y| x.bitxor_assign(y))
+            }
+
+            _ => Err(DataFusionError::NotImplemented(format!(
+                "GroupsAccumulator not supported for {} with {}",
+                self.name(),
+                self.data_type
+            ))),
+        }
     }
 
     fn reverse_expr(&self) -> Option<Arc<dyn AggregateExpr>> {
@@ -694,65 +600,6 @@ impl Accumulator for BitXorAccumulator {
     }
 }
 
-#[derive(Debug)]
-struct BitXorRowAccumulator {
-    index: usize,
-    datatype: DataType,
-}
-
-impl BitXorRowAccumulator {
-    pub fn new(index: usize, datatype: DataType) -> Self {
-        Self { index, datatype }
-    }
-}
-
-impl RowAccumulator for BitXorRowAccumulator {
-    fn update_batch(
-        &mut self,
-        values: &[ArrayRef],
-        accessor: &mut RowAccessor,
-    ) -> Result<()> {
-        let values = &values[0];
-        let delta = &bit_xor_batch(values)?;
-        bit_xor_row(self.index, accessor, delta)?;
-        Ok(())
-    }
-
-    fn update_scalar_values(
-        &mut self,
-        values: &[ScalarValue],
-        accessor: &mut RowAccessor,
-    ) -> Result<()> {
-        let value = &values[0];
-        bit_xor_row(self.index, accessor, value)
-    }
-
-    fn update_scalar(
-        &mut self,
-        value: &ScalarValue,
-        accessor: &mut RowAccessor,
-    ) -> Result<()> {
-        bit_xor_row(self.index, accessor, value)
-    }
-
-    fn merge_batch(
-        &mut self,
-        states: &[ArrayRef],
-        accessor: &mut RowAccessor,
-    ) -> Result<()> {
-        self.update_batch(states, accessor)
-    }
-
-    fn evaluate(&self, accessor: &RowAccessor) -> Result<ScalarValue> {
-        Ok(accessor.get_as_scalar(&self.datatype, self.index))
-    }
-
-    #[inline(always)]
-    fn state_index(&self) -> usize {
-        self.index
-    }
-}
-
 /// Expression for a BIT_XOR(DISTINCT) aggregation.
 #[derive(Debug, Clone)]
 pub struct DistinctBitXor {
