@@ -20,6 +20,7 @@
 use ahash::RandomState;
 use std::any::Any;
 use std::fmt::Debug;
+use std::hash::{Hash, Hasher};
 use std::sync::Arc;
 
 use crate::hash_utils::HashValue;
@@ -33,9 +34,7 @@ use arrow::record_batch::RecordBatch;
 use arrow::util::bit_iterator::BitIndexIterator;
 use arrow::{downcast_dictionary_array, downcast_primitive_array};
 use datafusion_common::{
-    cast::{
-        as_boolean_array, as_generic_binary_array, as_primitive_array, as_string_array,
-    },
+    cast::{as_boolean_array, as_generic_binary_array, as_string_array},
     DataFusionError, Result, ScalarValue,
 };
 use datafusion_expr::ColumnarValue;
@@ -178,14 +177,6 @@ fn make_set(array: &dyn Array) -> Result<Arc<dyn Set>> {
             let array = as_boolean_array(array)?;
             Arc::new(ArraySet::new(array, make_hash_set(array)))
         },
-        DataType::Decimal128(_, _) => {
-            let array = as_primitive_array::<Decimal128Type>(array)?;
-            Arc::new(ArraySet::new(array, make_hash_set(array)))
-        }
-        DataType::Decimal256(_, _) => {
-            let array = as_primitive_array::<Decimal256Type>(array)?;
-            Arc::new(ArraySet::new(array, make_hash_set(array)))
-        }
         DataType::Utf8 => {
             let array = as_string_array(array)?;
             Arc::new(ArraySet::new(array, make_hash_set(array)))
@@ -330,6 +321,14 @@ impl PhysicalExpr for InListExpr {
             self.static_filter.clone(),
         )))
     }
+
+    fn dyn_hash(&self, state: &mut dyn Hasher) {
+        let mut s = state;
+        self.expr.hash(&mut s);
+        self.negated.hash(&mut s);
+        self.list.hash(&mut s);
+        // Add `self.static_filter` when hash is available
+    }
 }
 
 impl PartialEq<dyn Any> for InListExpr {
