@@ -15,13 +15,15 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use crate::equivalence::OrderingEquivalenceBuilder;
-use crate::PhysicalExpr;
+use crate::{PhysicalExpr, PhysicalSortExpr};
+
 use arrow::array::ArrayRef;
 use arrow::datatypes::Field;
 use arrow::record_batch::RecordBatch;
+use arrow_schema::SchemaRef;
 use datafusion_common::Result;
 use datafusion_expr::PartitionEvaluator;
+
 use std::any::Any;
 use std::sync::Arc;
 
@@ -80,9 +82,11 @@ pub trait BuiltInWindowFunctionExpr: Send + Sync + std::fmt::Debug {
         None
     }
 
-    /// Adds any equivalent orderings generated by this expression
-    /// to `builder`.
-    ///
-    /// The default implementation does nothing
-    fn add_equal_orderings(&self, _builder: &mut OrderingEquivalenceBuilder) {}
+    /// Returns the ordering introduced by the window function, if applicable.
+    /// Most window functions don't introduce an ordering, hence the default
+    /// value is `None`. Note that this information is used to update ordering
+    /// equivalences.
+    fn get_result_ordering(&self, _schema: &SchemaRef) -> Option<PhysicalSortExpr> {
+        None
+    }
 }
