@@ -269,9 +269,9 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                 negated,
             } => self.sql_in_list_to_expr(*expr, list, negated, schema, planner_context),
 
-            SQLExpr::Like { negated, expr, pattern, escape_char } => self.sql_like_to_expr(negated, *expr, *pattern, escape_char, schema, planner_context),
+            SQLExpr::Like { negated, expr, pattern, escape_char } => self.sql_like_to_expr(negated, *expr, *pattern, escape_char, schema, planner_context,false),
 
-            SQLExpr::ILike { negated, expr, pattern, escape_char } =>  self.sql_ilike_to_expr(negated, *expr, *pattern, escape_char, schema, planner_context),
+            SQLExpr::ILike { negated, expr, pattern, escape_char } =>  self.sql_like_to_expr(negated, *expr, *pattern, escape_char, schema, planner_context,true),
 
             SQLExpr::SimilarTo { negated, expr, pattern, escape_char } => self.sql_similarto_to_expr(negated, *expr, *pattern, escape_char, schema, planner_context),
 
@@ -389,6 +389,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         )))
     }
 
+    #[allow(clippy::too_many_arguments)]
     fn sql_like_to_expr(
         &self,
         negated: bool,
@@ -397,6 +398,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         escape_char: Option<char>,
         schema: &DFSchema,
         planner_context: &mut PlannerContext,
+        case_insensitive: bool,
     ) -> Result<Expr> {
         let pattern = self.sql_expr_to_logical_expr(pattern, schema, planner_context)?;
         let pattern_type = pattern.get_type(schema)?;
@@ -410,30 +412,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
             Box::new(self.sql_expr_to_logical_expr(expr, schema, planner_context)?),
             Box::new(pattern),
             escape_char,
-        )))
-    }
-
-    fn sql_ilike_to_expr(
-        &self,
-        negated: bool,
-        expr: SQLExpr,
-        pattern: SQLExpr,
-        escape_char: Option<char>,
-        schema: &DFSchema,
-        planner_context: &mut PlannerContext,
-    ) -> Result<Expr> {
-        let pattern = self.sql_expr_to_logical_expr(pattern, schema, planner_context)?;
-        let pattern_type = pattern.get_type(schema)?;
-        if pattern_type != DataType::Utf8 && pattern_type != DataType::Null {
-            return Err(DataFusionError::Plan(
-                "Invalid pattern in ILIKE expression".to_string(),
-            ));
-        }
-        Ok(Expr::ILike(Like::new(
-            negated,
-            Box::new(self.sql_expr_to_logical_expr(expr, schema, planner_context)?),
-            Box::new(pattern),
-            escape_char,
+            case_insensitive,
         )))
     }
 
@@ -458,6 +437,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
             Box::new(self.sql_expr_to_logical_expr(expr, schema, planner_context)?),
             Box::new(pattern),
             escape_char,
+            false,
         )))
     }
 
